declare filename "BLIT_pulse_train.dsp"; declare name "BLIT_pulse_train"; ba = library("basics.lib");
ma = library("maths.lib");
de = library("delays.lib");
fi = library("filters.lib");
si = library("signals.lib");

// Nyquist
ny = ma.SR/2;

// phasor with phase reset
ph(freq, reset) = freq/ma.SR : (+ : ma.decimal) ~ *(1-(reset : !=(0)));

// differentiator
diff(x) = x-x';

// NAN-safe divider
div(x1, x2) = ba.if(x2 : ==(0), 0, (x1, x2 : /));

// first derivative
delta(s, t, in) = in, del_lin(s, t, in) : -;

// linear interpolation between samples
del_lin(size, del, in) = de.fdelay(size*ma.SR, del*ma.SR, in);

// unipolar BLIT
blit_up(f, h) = ba.if(phase : ==(0), 1, sinc)
with {
      sinc =   sin(phase*h_1), sin(phase) : div, h_1 : div;
      phase = ph(f_1, 0)*2*ma.PI;
      f_1 = f/2;
      h_1 = diff(phase) : <(0),
            (h*2+1 : min(lim) : int) : ba.sAndH;
      lim = ba.if((int((ma.SR, f : div)) : %(2) : ==(0)), int((ma.SR, f : div))-1, int((ma.SR, f : div)));
};

// bipolar BLIT
blit_bp(f, h, d) = delta(10, d1, blit_up(f, h))
with {
      d1 = d*( 1, f : div);
};

pulse_train(f, h, d) = blit_bp(f, h, d) : fi.pole(1-10/ma.SR) : *(scale) : +(d)
with {
      lim =    ny,
               f : div;
      scale =  (h : min(lim)), 
               lim : div;
};

f = hslider("freq", 135, 0.0001, 10000, 0.0001);
h = hslider("h", 5, 0.5, 100, 0.001);
d = hslider("d", 1.3, 0., 5., 0.001):si.smooth(0.999);

process = pulse_train(f, h, d) : _;