declare filename "harmonicExciter.dsp"; declare name "harmonicExciter"; declare exciter_name "harmonicExciter";
declare exciter_author "Priyanka Shekar (pshekar@ccrma.stanford.edu), revised by RM";
declare exciter_copyright "Copyright (c) 2013 Priyanka Shekar";
declare exciter_version "1.0";
declare exciter_license "MIT License (MIT)";
declare description "Psychoacoustic harmonic exciter, with GUI";

//import("stdfaust.lib");
ma = library("maths.lib");
ba = library("basics.lib");
si = library("signals.lib");

//singleprecision EPSILON = 1.192092896e-07;
//doubleprecision EPSILON = 2.2204460492503131e-016;
//quadprecision EPSILON = 2.2204460492503131e-016;
EPSILON = 2.2204460492503131e-016;

amp_follower_ar(att,rel) = abs : lag_ud(att,rel);

lag_ud(up,dn) = _ <: ((>,ba.tau2pole(up),tau2pole(dn):select2),_:si.smooth) ~ _;

tau2pole(tau) = ba.if(clipCond, 0.0, exp(-1.0/(tauCenterClipped*float(ma.SR)))) with {
 clipCond = abs(tau)<EPSILON;
 tauCenterClipped = ba.if(clipCond, 1.0, tau); // 1.0 can be any nonzero value (not used)
};

bypass1(bpc,e) = _ <: select2(bpc,(inswitch:e),_)
with {
    inswitch = select2(bpc,_,0);
};

tf2(b0,b1,b2,a1,a2) = sub ~ conv2(a1,a2) : conv3(b0,b1,b2)
with {
	conv3(k0,k1,k2,x) = k0*x + k1*x' + k2*x'';
	conv2(k0,k1,x)    = k0*x + k1*x';
	sub(x,y)          = y-x;
};

tf2s(b2,b1,b0,a1,a0,w1) = tf2(b0d,b1d,b2d,a1d,a2d)
with {
  c   = 1/tan(w1*0.5/ma.SR); // bilinear-transform scale-factor
  csq = c*c;
  d   = a0 + a1 * c + csq;
  b0d = (b0 + b1 * c + b2 * csq)/d;
  b1d = 2 * (b0 - b2 * csq)/d;
  b2d = (b0 - b1 * c + b2 * csq)/d;
  a1d = 2 * (a0 - csq)/d;
  a2d = (a0 - a1*c + csq)/d;
};

lowpass0_highpass1(s,N,fc) = lphpr(s,N,N,fc)
with {
  lphpr(s,0,N,fc) = _;
  lphpr(s,1,N,fc) = tf1s(s,1-s,1,2*ma.PI*fc);
  lphpr(s,O,N,fc) = lphpr(s,(O-2),N,fc) : tf2s(s,0,1-s,a1s,1,w1) with {
    parity = N % 2;
    S = (O-parity)/2; // current section number
    a1s = -2*cos((ma.PI)*-1 + (1-parity)*ma.PI/(2*N) + (S-1+parity)*ma.PI/N);
    w1 = 2*ma.PI*fc;
  };
};

compression_gain_mono(ratio,thresh,att,rel) =
  amp_follower_ar(att,rel) : ba.linear2db : outminusindb(ratio,thresh) :
  kneesmooth(att) : ba.db2linear
with {
  // kneesmooth(att) installs a "knee" in the dynamic-range compression,
  // where knee smoothness is set equal to half that of the compression-attack.
  // A general 'knee' parameter could be used instead of tying it to att/2:
  kneesmooth(att) = si.smooth(tau2pole(att/2.0));
  // compression gain in dB:
   outminusindb(ratio,thresh,level) = max(level-thresh,0.0) * (1.0/float(ratio)-1.0);
  // Note: "float(ratio)" REQUIRED when ratio is an integer > 1!
};

compressor_mono(ratio,thresh,att,rel,x) = x * compression_gain_mono(ratio,thresh,att,rel,x);
highpass(N,fc) = lowpass0_highpass1(1,N,fc);

exciter = _ <: (highpass(2, fc) : compressor : pregain : harmonicCreator :
	postgain), _ : balance
with{
	compressor = bypass1(cbp,compressorMono)
	with{
		comp_group(x) = vgroup("COMPRESSOR", x);

	meter_group(x)	= comp_group(hgroup("[0]", x));
	knob_group(x)  = comp_group(hgroup("[1]", x));

	cbp = meter_group(checkbox("bypass"));

	gainview = compression_gain_mono(ratio,threshold,attack,release) : ba.linear2db
		: hbargraph("Compressor_Gain",-50,+10);

	displaygain = _ <: _,abs : _,gainview : attach;

	compressorMono = displaygain(compressor_mono(ratio,threshold,attack,release));

	ratio = hslider("ratio [style:knob]", 5, 1, 20, 0.1);
	threshold = hslider("threshold [unit:dB] [style:knob]", -30, -100, 10, 0.1);
	attack = hslider("attack [unit:ms] [style:knob]", 50, 0, 500, 0.1) : *(0.001) : max(1/ma.SR);
	release = hslider("release [unit:ms] [style: knob]", 500, 0, 1000, 0.1) : *(0.001) : max(1/ma.SR);
	};

	//Exciter GUI controls
	//Highpass - selectable cutoff frequency
	fc = hslider("cutoff [unit:Hz] [style:knob] [scale:log]", 5000, 1000, 10000, 100);

	//Pre-distortion gain - selectable percentage of harmonics
	ph = hslider("harmonics [unit:percent] [style:knob]", 20, 0, 200, 1) / 100;
	pregain = * (ph);

	//Asymmetric cubic soft clipper
	harmonicCreator(x) = x <: cubDist1, cubDist2, cubDist3 :> _;
	cubDist1(x) = (x < 0) * x;
	cubDist2(x) = (x >= 0) * (x <= 1) * (x - x ^ 3 / 3);
	cubDist3(x) = (x > 1) * 2/3;

	//Post-distortion gain - undoes effect of pre-gain
	postgain = * (1/ph);

	//Balance - selectable dry/wet mix
	ml = hslider("mix [style:knob]", 0.50, 0.00, 1.00, 0.01);
	balance = (_ * ml), (_ * (1.0 - ml)) :> _;
};

process = exciter;